<?php

namespace DHLParcel\Shipping\Controller\BackOffice;

use DHLParcel\Shipping\Model\Api\Connector;
use DHLParcel\Shipping\Model\Core\TranslateTrait;
use DHLParcel\Shipping\Model\Core\Kernel;
use DHLParcel\Shipping\Model\Core\Settings;
use DHLParcel\Shipping\Model\Data\Api\Response\Capability\Option;
use DHLParcel\Shipping\Model\Data\Db\OrderData\ServicePoint as ServicePointOrderData;
use DHLParcel\Shipping\Model\Data\Preset;
use DHLParcel\Shipping\Model\Data\Capability\Set as CapabilitySet;
use DHLParcel\Shipping\Model\Service\BulkProfileService;
use DHLParcel\Shipping\Model\Service\CapabilityService;
use DHLParcel\Shipping\Model\Service\LabelService;
use DHLParcel\Shipping\Model\Service\OrderService;
use DHLParcel\Shipping\Model\Service\PanelNotificationService;
use DHLParcel\Shipping\Model\Service\PanelService;
use DHLParcel\Shipping\Model\Service\PresetService;
use DHLParcel\Shipping\Model\Service\ServicePointService;
use DHLParcel\Shipping\Model\Service\SettingsService;
use DHLParcel\Shipping\Model\Service\ShipmentService;
use PrestaShopBundle\Controller\Admin\FrameworkBundleAdminController;
use Symfony\Component\HttpFoundation\JsonResponse;
use Tools;
use Order;
use Address;
use Country;
use Configuration;
use Context;

class AjaxController extends FrameworkBundleAdminController
{
    use TranslateTrait;

    public function panelCreateAction()
    {
        $orderId = Tools::getValue('order_id');
        $business = boolval(Tools::getValue('business') == '1');
        $selectedOptions = Tools::getValue('selected_options');
        $selectedOptionsData = Tools::getValue('selected_options_data');
        $size = Tools::getValue('size');

        $shipmentService = ShipmentService::instance();

        if (!is_array($selectedOptionsData)) {
            $selectedOptionsData = [];
        }

        $success = $shipmentService->create($orderId, $size, $selectedOptions, $selectedOptionsData, $business);
        if ($success) {
            PanelNotificationService::instance()->addNotice($this->l('Label created.'));
        } else {
            PanelNotificationService::instance()->addError($this->l('Label could not be created.'));
            if ($errorMessage = $shipmentService->getError(ShipmentService::CREATE_ERROR)) {
                PanelNotificationService::instance()->addError($errorMessage);
            }
        }

        $view = [];
        $view['notifications'] = $this->renderTemplate('backoffice/order/panel/notifications', [
            'notifications' => [
                'notices'  => PanelNotificationService::instance()->getNotices(),
                'warnings' => PanelNotificationService::instance()->getWarnings(),
                'errors'   => PanelNotificationService::instance()->getErrors(),
            ],
        ]);

        $labels = LabelService::instance()->getAll($orderId);
        $view['labels'] = $this->renderTemplate('backoffice/order/panel/labels', [
            'labels' => $labels,
        ]);

        $response = new JsonResponse();
        $response->setData([
            'status'  => $success ? 'success' : 'error',
            'data'    => [
                'view' => $view,
                'extra' => $shipmentService->lastTrackingNumber ? [
                    'tracking_number' => $shipmentService->lastTrackingNumber
                ] : null,
            ],
            'message' => null,
        ]);

        return $response;
    }

    public function panelDeleteAction()
    {
        $labelId = Tools::getValue('label_id');
        $label = LabelService::instance()->load($labelId);

        if ($label) {
            $success = LabelService::instance()->delete($labelId);

            if ($success) {
                PanelNotificationService::instance()->addNotice($this->l('Label deleted.'));
            } else {
                PanelNotificationService::instance()->addError($this->l('Label could not be deleted.'));
            }
        } else {
            PanelNotificationService::instance()->addError($this->l('Label could not be found.'));
            $success = false;
        }

        $view = [];
        $view['notifications'] = $this->renderTemplate('backoffice/order/panel/notifications', [
            'notifications' => [
                'notices'  => PanelNotificationService::instance()->getNotices(),
                'warnings' => PanelNotificationService::instance()->getWarnings(),
                'errors'   => PanelNotificationService::instance()->getErrors(),
            ],
        ]);

        if ($label) {
            $labels = LabelService::instance()->getAll($label->id_order);
            $view['labels'] = $this->renderTemplate('backoffice/order/panel/labels', [
                'labels' => $labels,
            ]);
        } else {
            $view['labels'] = null;
        }

        $response = new JsonResponse();
        $response->setData([
            'status'  => $success ? 'success' : 'error',
            'data'    => [
                'view' => $view,
            ],
            'message' => null,
        ]);

        return $response;
    }

    public function panelAction()
    {
        $level = intval(Tools::getValue('level'));
        $orderId = Tools::getValue('order_id');
        $business = boolval(Tools::getValue('business') == '1');
        $selectedOptions = Tools::getValue('selected_options') ?: [];
        $selectedOptionsData = Tools::getValue('selected_options_data') ?: [];

        if ($level == 3) {
            $capabilityOptions = [];
        } else {
            $capabilityOptions = $selectedOptions;
        }

        $capabilities = CapabilityService::instance()->check(
            $this->getReceiverCountry($orderId),
            $this->getReceiverPostcode($orderId),
            $business,
            $capabilityOptions
        );

        $view = [];
        if ($level >= 2) {
            $options = CapabilityService::instance()->getOptions($capabilities);
            $deliveryOptions = PanelService::instance()->parseDeliveryOptions($options, $selectedOptions, $selectedOptionsData);
            $serviceOptions = PanelService::instance()->parseServiceOptions($options, $selectedOptions, $selectedOptionsData);

            $view['delivery_options'] = $this->renderTemplate('backoffice/order/panel/delivery-options', [
                'deliveryOptions' => $deliveryOptions,
            ]);

            $view['service_options'] = $this->renderTemplate('backoffice/order/panel/service-options', [
                'serviceOptions' => $serviceOptions,
            ]);
        } else {
            $view['delivery_options'] = null;
            $view['service_options'] = null;
        }

        if ($level >= 1) {
            $selectedOption = null;
            if ($level == 3 && !empty($deliveryOptions)) {
                $selectedOption = reset($deliveryOptions)->key;
            }

            $sizes = CapabilityService::instance()->getSizes($capabilities, $selectedOption);
            $sizes = CapabilityService::instance()->addDisplayKgs($sizes);
            $view['sizes'] = $this->renderTemplate('backoffice/order/panel/sizes', [
                'sizes' => $sizes,
            ]);
        } else {
            $view['sizes'] = null;
        }

        $response = new JsonResponse();
        $response->setData([
            'status'  => 'success',
            'data'    => [
                'view' => $view,
            ],
            'message' => null,
        ]);

        return $response;
    }

    public function panelServicePointAction()
    {
        $orderId = Tools::getValue('order_id');
        $search = Tools::getValue('search');

        $country = $this->getReceiverCountry($orderId);

        $servicePoints = ServicePointService::instance()->search($search, $country);

        $view = [];
        $view['list'] = $this->renderTemplate('backoffice/order/panel/input/servicepoint/list', [
            'servicePoints' => $servicePoints
        ]);

        $response = new JsonResponse();
        $response->setData([
            'status'  => 'success',
            'data'    => [
                'view' => $view,
            ],
            'message' => null,
        ]);

        return $response;
    }

    public function carrierAction()
    {
        $shippingCountry = SettingsService::instance()->shippingCountry();
        $capabilities = CapabilityService::instance()->check($shippingCountry);

        $options = CapabilityService::instance()->getOptions($capabilities);
        $presetOptions = PresetService::instance()->options();
        foreach ($options as $key => $option) {
            if (!in_array($key, $presetOptions)) {
                $options[$key] = null;
                unset($options[$key]);
            }
        }

        $carrierId = Tools::getValue('carrier_id');
        $preset = PresetService::instance()->load($carrierId);

        $deliveryOptions = PanelService::instance()->parseDeliveryOptions($options, $preset->options);
        $serviceOptions = PanelService::instance()->parseServiceOptions($options, $preset->options);

        $presetData = new Preset();

        $presetData->carrierId = Tools::getValue('carrier_id');
        $presetData->link = boolval($preset->link);
        $presetData->deliveryOptions = $deliveryOptions;
        $presetData->serviceOptions = $serviceOptions;

        $view = [];
        $view['form'] = $this->renderTemplate('backoffice/carrier/form', $presetData->toArray());

        $response = new JsonResponse();
        $response->setData([
            'status'  => !empty($view) ? 'success' : 'error',
            'data'    => [
                'view' => $view,
            ],
            'message' => null,
        ]);
        return $response;
    }

    public function carrierTemporarySaveAction()
    {
        $carrierId = Tools::getValue('carrier_id');
        $link = Tools::getValue('link');
        $selectedOptions = Tools::getValue('selected_options');

        $success = PresetService::instance()->update($carrierId, $link, $selectedOptions, true);

        $response = new JsonResponse();
        $response->setData([
            'status'  => $success ? 'success' : 'error',
            'data'    => $success,
            'message' => null,
        ]);
        return $response;
    }

    public function carrierResetAction()
    {
        PresetService::instance()->deleteCarriers();
        PresetService::instance()->addCarriers();

        $response = new JsonResponse();
        $response->setData([
            'status'  => 'success',
            'data'    => [],
            'message' => $this->l('DHL shipping methods are reset.'),
        ]);

        return $response;
    }

    public function authenticateAction()
    {
        $apiUserId = Tools::getValue('user_id');
        $apiKey = Tools::getValue('key');

        $accounts = Connector::instance()->testAuthenticate($apiUserId, $apiKey);

        $response = new JsonResponse();
        $response->setData([
            'status'  => !empty($accounts) ? 'success' : 'error',
            'data'    => [
                'accounts' => $accounts,
            ],
            'message' => !empty($accounts) ? $this->l('Connection successful') : $this->l('Authentication failed'),
        ]);

        return $response;
    }

    public function bulkProfileTemplateAction()
    {
        $shippingCountry = Configuration::get(Settings::SHIPPING_ADDRESS_COUNTRY);
        $countries = Country::getCountries(Context::getContext()->cookie->id_lang, true);

        // Create sets
        $consumerCapabilities = CapabilityService::instance()->check('', '', false);
        $businessCapabilities = CapabilityService::instance()->check('', '', true);
        $defaultCapabilities = array_merge($consumerCapabilities, $businessCapabilities);

        $defaultSet = $this->getCapabilitySet($defaultCapabilities);

        $view = $this->renderTemplate('backoffice/settings/bulk/profiles', [
            'shippingCountry' => $shippingCountry,
            'countries' => $countries,
            'deliveryOptions' => $defaultSet->deliveryOptions,
            'serviceOptions' => $defaultSet->serviceOptions,
            'sizes' => $defaultSet->sizes,
        ]);

        $response = new JsonResponse();
        $response->setData([
            'status'  => !empty($view) ? 'success' : 'error',
            'data'    => [
                'view' => $view,
            ],
            'message' => null,
        ]);
        return $response;
    }

    public function bulkProfileSearchAction()
    {
        $receiverCountry = Tools::getValue('receiverCountry') ?: '';
        switch (Tools::getValue('shipmentType')) {
            case 'business':
                $toBusiness = true;
                break;
            case 'private':
                $toBusiness = false;
                break;
            default:
                $toBusiness = null;
        }

        $deliveryTypeValue = Tools::getValue('deliveryType');
        $deliveryOptions = $deliveryTypeValue ? [$deliveryTypeValue] : [];

        // Create set
        $capabilities = CapabilityService::instance()->check($receiverCountry, '', $toBusiness, $deliveryOptions);
        $set = $this->getCapabilitySet($capabilities);
        $set->serviceOptions = $this->filterOutExclusions($set->serviceOptions, $deliveryOptions);

        $deliveryTypeView = $this->renderTemplate('backoffice/settings/bulk/profiles/delivery-type', [
            'deliveryOptions' => $set->deliveryOptions,
        ]);

        $serviceOptionsView = $this->renderTemplate('backoffice/settings/bulk/profiles/service-options', [
            'serviceOptions' => $set->serviceOptions,
        ]);

        $sizesView = $this->renderTemplate('backoffice/settings/bulk/profiles/sizes', [
            'sizes' => $set->sizes,
        ]);

        $response = new JsonResponse();
        $response->setData([
            'status'  => 'error',
            'data'    => [
                'view' => [
                    'deliveryType' => $deliveryTypeView,
                    'serviceOptions' => $serviceOptionsView,
                    'sizes' => $sizesView,
                ],
            ],
            'message' => null,
        ]);
        return $response;
    }

    public function bulkDownloadAction()
    {
        $orderIds = Tools::getValue('order_ids') ?: [];

        $labels = [];
        foreach ($orderIds as $orderId) {
            $labels = array_merge($labels, LabelService::instance()->getAll($orderId));
        }

        $labelIds = [];
        foreach ($labels as $label) {
            $labelIds[] = $label->label_uuid;
        }

        $file = LabelService::instance()->requestMultiPDF($labelIds);

        $response = new JsonResponse();
        $response->setData([
            'status'  => $file ? 'success' : 'error',
            'data'    => [
                'url' => $file ? Tools::getHttpHost(true) . __PS_BASE_URI__ . 'upload/' . $file : null
            ],
            'message' => $file ? $this->l('Generated file') : $this->l('Cannot generate file'),
        ]);

        return $response;
    }

    public function bulkCreateAction()
    {
        $errorMessage = '';
        $code = Tools::getValue('code');
        $orderIds = Tools::getValue('order_ids') ?: [];

        $bulkProfile = BulkProfileService::instance()->load($code);
        $requiredOptions = $bulkProfile->required_options;
        $skippableOptions = array_diff($bulkProfile->options, $bulkProfile->required_options);

        $successCount = 0;
        $failCount = 0;

        if ($bulkProfile->business === 'private') {
            $business = false;
        } elseif ($bulkProfile->business === 'business') {
            $business = true;
        } else {
            $business = boolval(Configuration::get(Settings::BUSINESS));
        }

        foreach ($orderIds as $orderId) {
            // We never overwrite ServicePoint selections (for now)
            $order = new Order($orderId);
            $preset = PresetService::instance()->load($order->id_carrier);
            $selectedOptionsData = [];
            $selectedOptions = [];

            if ($preset && in_array(Option::KEY_PS, $preset->options)) {
                // Check for saved data from custom table
                $servicePointOrderData = OrderService::instance()->getDataKey($orderId, OrderService::SERVICEPOINT);
                $servicePointData = new ServicePointOrderData($servicePointOrderData);
                if ($servicePointData->servicePointId) {
                    $selectedOptions = [Option::KEY_PS];
                    $selectedOptionsData[Option::KEY_PS] = $servicePointData->servicePointId;
                }
            } elseif ($bulkProfile->delivery === 'BP') {
                $selectedOptions = [Option::KEY_BP];
            } elseif ($preset) {
                // Use default
                $selectedOptions = $preset->options;
            }

            $requestOptions = array_merge($requiredOptions, $selectedOptions);
            $allowedData = $this->getAllowedData($orderId, $requestOptions, $business);

            $autoEnabledOptions = PanelService::instance()->autoEnabledOptions($orderId);
            $requestOptions = array_merge($requestOptions, array_keys($autoEnabledOptions));
            $requestOptionsData = array_merge($selectedOptionsData, $autoEnabledOptions);

            foreach ($skippableOptions as $skippableOption) {
                if (array_key_exists($skippableOption, $allowedData['allowed_options'])) {
                    $requestOptions[] = $skippableOption;
                }
            }

            if ($bulkProfile->size === 'smallest') {
                $size = !empty($allowedData['allowed_sizes']) ? reset($allowedData['allowed_sizes'])->key : null;
            } elseif ($bulkProfile->size === 'largest') {
                $size = !empty($allowedData['allowed_sizes']) ? end($allowedData['allowed_sizes'])->key : null;
            } else {
                $size = $bulkProfile->size;
            }

            $shipmentService = ShipmentService::instance();
            $success = $shipmentService->create($orderId, $size, $requestOptions, $requestOptionsData, $business);

            if (!$success) {
                $errorMessage .= '<br /><br />';
                $errorMessage .= 'Error OrderID #' . $orderId . ':<br />';
                $errorMessage .= nl2br($shipmentService->getError(ShipmentService::CREATE_ERROR));
            }

            if ($success) {
                $successCount++;
            } else {
                $failCount++;
            }
        }

        $response = new JsonResponse();
        $response->setData([
            'status'  => $successCount ? 'success' : 'error',
            'data'    => [
                'success' => $successCount,
                'fail' => $failCount,
                'error_message' => preg_replace("/\r|\n/", "", $errorMessage),
            ],
            'message' => $successCount ? $this->l('Created labels') : $this->l('Cannot create any labels'),
        ]);

        return $response;
    }

    protected function getAllowedData($orderId, $requestedOptions, $business)
    {
        $order = new Order($orderId);

        $receiverAddress = new Address($order->id_address_delivery);
        $capabilities = CapabilityService::instance()->check(
            Country::getIsoById($receiverAddress->id_country),
            $receiverAddress->postcode,
            $business,
            $requestedOptions
        );

        $options = CapabilityService::instance()->getOptions($capabilities);
        $options = $this->filterOutExclusions($options, $requestedOptions);

        $sizes = CapabilityService::instance()->getSizes($capabilities);

        return [
            'order' => $order,
            'allowed_options' => $options,
            'allowed_sizes' => $sizes
        ];
    }

    protected function getReceiverCountry($orderId)
    {
        $order = new Order($orderId);
        $receiverAddress = new Address($order->id_address_delivery);
        return strtoupper(Country::getIsoById($receiverAddress->id_country));
    }

    protected function getReceiverPostcode($orderId)
    {
        $order = new Order($orderId);
        $receiverAddress = new Address($order->id_address_delivery);
        return strtoupper($receiverAddress->postcode);
    }

    protected function renderTemplate($path, $data = [])
    {
        if ($this->container && $this->container->get('twig')) {
            if (version_compare(_PS_VERSION_, '1.7.5', '>=')) {
                $template = '@Modules/dhlparcel_shipping/views/templates/' . $path . '.twig';
            } elseif (version_compare(_PS_VERSION_, '1.7.0', '>=') && version_compare(_PS_VERSION_, '1.7.5', '<')) {
                $template = _PS_MODULE_DIR_ . 'dhlparcel_shipping/views/templates/' . $path . '.twig';
            } else {
                return false;
            }
            return $this->container->get('twig')->render($template, $data);
        } else {
            return Kernel::instance()->render($path, $data);
        }
    }

    /**
     * @param $capabilities
     * @return CapabilitySet
     */
    protected function getCapabilitySet($capabilities)
    {
        $options = CapabilityService::instance()->getOptions($capabilities);
        $bulkOptions = BulkProfileService::instance()->options();
        foreach ($options as $key => $option) {
            if (!in_array($key, $bulkOptions)) {
                unset($options[$key]);
            }
        }

        return new CapabilitySet([
            'deliveryOptions' => PanelService::instance()->parseDeliveryOptions($options),
            'serviceOptions' => PanelService::instance()->parseServiceOptions($options),
            'sizes' => CapabilityService::instance()->getSizes($capabilities),
        ]);
    }

    protected function filterOutExclusions($options, $filterOutOptions)
    {
        if (!is_array($options)) {
            return [];
        }
        $excludedOptions = [];
        // Manually filter out options that lists current requested options as excluded, as the API does not do that.
        foreach ($options as $key => $option) {
            foreach ($option->exclusions as $exclusion) {
                if (in_array($exclusion->key, $filterOutOptions)) {
                    $excludedOptions[] = $key;
                }
            }
        }
        foreach (array_unique($excludedOptions) as $excludedOption) {
            $options[$excludedOption] = null;
            unset($options[$excludedOption]);
        }
        return $options;
    }
}
